C#######################################################################
C  File rect2pol.f
C  Map from rectangular to polar coordinates.
C  Author: Dave Nichols 
C  Taken from  Mihai Popovici's TTmap code based on  Jos Van Trier's code.
C
C x_shot                  = shot X coordinate
C z_shot                  = shot Z coordinate
C nz                      = number of depth levels on the grid
C o_x,d_x                 = left X coordinate on the grid and increment
C nx                      = number of X points on the grid
C o_z,d_z                 = first depth level and depth increments
C n_theta                 = number of angles for the grid
C o_theta,d_theta         = first angle and angle increments (reset by conj=0)
C nr                      = number of radius increments
C o_r,d_r                 = first radius and radius increments (reset by conj=0)
C
C rect_data(nx,nz)             = data in rectangular coordinates.
C polar_data(0:n_theta+1,nr)    = data in polar coordinates.
C 
C conj                    = ( 0=rect->polar, 1=polar->rect )
C 
C
C#######################################################################
        subroutine rect2pol(x_shot,z_shot, nx,o_x,d_x, nz,o_z,d_z,
     &  n_theta,o_theta,d_theta, nr,o_r,d_r, rect_data,polar_data, conj )
	implicit none
        integer nx,nz,nr,n_theta, conj
        real rect_data(nx,nz)
        real polar_data(n_theta,nr)
        real x_shot,z_shot,o_x,d_x,o_z,d_z,o_theta,d_theta,o_r
        real d_r
        real PI,eps,epsilon,xmax,zmax,rmax,rr,xx,zz,theta,ddx,ddz,ddr
        real xmax2,zmax2
        real a11,a12,a21,a22,th,ddt
        integer it,ir,ix,iz
	real bilint2
        real bilint
        bilint(ddz,ddx,a11,a21,a12,a22) =
     &   (1-ddz)*(1-ddx)*a11+ddz*(1-ddx)*a21+
     &               (1-ddz)*ddx*a12+ddz*ddx*a22

C nr       = number of radius increments
C n_theta  = number of angle increments
C            usually nr=nz
C            usually ntheta=nx
C
        xmax=o_x+(nx-1)*d_x
        zmax=o_z+(nz-1)*d_z

C Check if shot coordinates are outside model
        if( (x_shot.lt.o_x.or.x_shot.gt.xmax) .or. 
     &  (z_shot.lt.o_z.or.z_shot.gt.zmax) ) then
          write(*,*) 'Shot coordinates are outside grid model'
          return
        endif
C Initialize constants
        PI = 4.*atan(1.)
        eps=0.001
        epsilon=1e-10
        o_r = 0.

       if( conj .eq. 0 ) then
C###########################################################################
C Map to cylindrical coordinates
C o_theta = initial angle, d_theta = angle increment


C Find the most distant corner from the shot location, that is the max. radius

        if( d_r .eq. 0 ) then
        xmax2=max(abs(x_shot-o_x),abs(xmax-x_shot))**2
        zmax2=max(abs(z_shot-o_z),abs(zmax-z_shot))**2
        rmax=sqrt(xmax2 + zmax2 )
        d_r =(rmax-o_r) / (nr-2)
        else
	rmax = o_r + (nr-1)*d_r
        endif
        

C        o_theta = -(100.-eps)*PI / 180.
C        d_theta = 2.*abs(o_theta)/(n_theta-2)
C        o_theta = o_theta - d_theta
        o_theta = -(90.)*PI / 180.
        d_theta = 2.*abs(o_theta)/(n_theta-1)

C Loop in Radius and Theta and express the slowness in cylindrical coordinates
        xmax=o_x+(nx-2)*d_x
        zmax=o_z+(nz-2)*d_z
        do ir=1,nr
          rr=o_r+(ir-1)*d_r
          do it=1,n_theta
            theta=o_theta+(it-1)*d_theta
            xx=rr*sin(theta)+x_shot
            zz=rr*cos(theta)+z_shot

C xx and zz stay within the boundaries (o_x,xmax) respectively (o_z,zmax)
            xx=max(o_x,xx)
            xx=min(xmax,xx)
            zz=max(o_z,zz)
            zz=min(zmax,zz)
C Find index in the (X,Z) grid
            ix=int((xx-o_x)/d_x)+1
            iz=int((zz-o_z)/d_z)+1

            ddx=(xx-o_x)/d_x-ix+1
            ddz=(zz-o_z)/d_z-iz+1
	    ix=min(ix,nx-1)
	    iz=min(iz,nz-1)

            a11=rect_data(ix,iz)
            a21=rect_data(ix,iz+1)
            a12=rect_data(ix+1,iz)
            a22=rect_data(ix+1,iz+1)
            polar_data(it,ir)=bilint(ddz,ddx,a11,a21,a12,a22)
          enddo
        enddo
      endif

      if( conj .eq. 1 ) then

C############################################################################
C Map to cartezian coordinates
        do iz=1,nz
          zz=o_z+(iz-1)*d_z-z_shot
          do ix=1,nx
            xx=o_x+(ix-1)*d_x-x_shot
            rr=sqrt(xx*xx+zz*zz)
            if(abs(zz).lt.epsilon) then
              th=sign(1.,xx)*PI/2.
            else
              th=atan2(xx,zz)
            endif
            it=int((th-o_theta)/d_theta)+1
            it=max(2,it)
            it=min(n_theta,it)
            ir=int((rr-o_r)/d_r)+1
            ir=max(1,ir)
            ir=min(nr-1,ir)

            ddr=(rr-o_r)/d_r-ir+1
            ddt=(th-o_theta)/d_theta-it+1
            a11=polar_data(it-1,ir)
            a21=polar_data(it,ir)
            a12=polar_data(it-1,ir+1)
            a22=polar_data(it,ir+1)
            rect_data(ix,iz)=bilint(ddt,ddr,a11,a21,a12,a22)
          enddo
        enddo
      endif

      return
      end

C###############################################################################
C  dx1   = Z distance to upper left corner
C  dx2   = X distance to upper left corner
C  a11   = Upper Left
C  a21   = Lower Left
C  a12   = Upper Right
C  a22   = Lower Right
C
C Interpolation is done using the function
C f(x,z)=A*x*z+B*x+C*z+D
C and solving the system f(0,0)=a11
C                        f(0,1)=a12
C                        f(1,0)=a21
C                        f(1,1)=a22
C###############################################################################
C        real function bilint2(dx1,dx2,a11,a21,a12,a22)
C        real dx1,dx2,a11,a21,a12,a22
C        bilint=(1-dx1)*(1-dx2)*a11+dx1*(1-dx2)*a21+
C     &               (1-dx1)*dx2*a12+dx1*dx2*a22
C        return
C        end


